import typing

from evntbus.event import Event


class Bus:
    def __init__(self):
        self.listeners = {}

    def listen(self, event: typing.Type, listener: typing.Callable,
               priority: int = 5):
        name = self._get_event_name(event)
        if name not in self.listeners:
            self.listeners[name] = {}
        if priority not in self.listeners[name]:
            self.listeners[name][priority] = []
        self.listeners[name][priority].append(listener)

    def emit(self, event: Event):
        name = self._get_event_name(type(event))
        if name not in self.listeners:
            return
        priorities = list(self.listeners[name].keys())
        priorities.sort(reverse=True)

        for priority in priorities:
            if priority not in self.listeners[name]:
                continue
            for listener in self.listeners[name][priority]:
                listener(event=event)

    @staticmethod
    def _get_event_name(event: typing.Type) -> str:
        return '.'.join([event.__module__, event.__name__])



import typing

if typing.TYPE_CHECKING:
    from evntbus.bus import Bus


def listen_decorator(evntbus: 'Bus'):
    class ListenDecorator(object):
        def __init__(self, event: typing.Type, priority: int = 5):
            self.event = event
            self.priority = priority

        def __call__(self, f: typing.Callable) -> typing.Callable:
            evntbus.listen(self.event, f, self.priority)
            return f

    return ListenDecorator



from distutils.core import setup

github = 'https://github.com/jmwri/evntbus'
version = '1.1.0'

setup(
    name='evntbus',
    packages=['evntbus'],
    version=version,
    license='MIT',
    python_requires='>=3.6, <4',
    description='A simple event bus.',
    author='Jim Wright',
    author_email='jmwri93@gmail.com',
    url=github,
    download_url='{github}/archive/{version}.tar.gz'.format(
        github=github, version=version
    ),
    keywords=['event', 'bus'],
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],
)
